// -*-c++-*-

/*
 *Copyright:

 Copyright (C) Hidehisa AKIYAMA

 This code is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.

 This code is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this code; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

 *EndCopyright:
 */

/////////////////////////////////////////////////////////////////////

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "bhv_basic_oneman_kick.h"

#include "body_kick_to_corner.h"

#include <rcsc/action/body_advance_ball.h>
//#include <rcsc/action/body_kick_multi_step.h>
#include <rcsc/action/body_kick_to_relative.h>
#include <rcsc/action/body_dribble.h>
#include <rcsc/action/body_hold_ball.h>
#include <rcsc/action/body_pass.h>
#include <rcsc/action/neck_scan_field.h>
#include <rcsc/action/neck_turn_to_low_conf_teammate.h>

#include <rcsc/player/player_agent.h>
#include <rcsc/player/debug_client.h>

#include <rcsc/common/logger.h>
#include <rcsc/common/server_param.h>
#include <rcsc/geom/sector_2d.h>

/*-------------------------------------------------------------------*/
/*!

*/
bool
Bhv_BasicOnemanKick::execute( rcsc::PlayerAgent * agent )
{

	std::cout << "step:" << agent->world().time() << std::endl;


	std::cout << "Oneman kick " << agent << std::endl;

    rcsc::dlog.addText( rcsc::Logger::TEAM,
                        "%s:%d: Bhv_BasicOnemanKick"
                        ,__FILE__, __LINE__ );

    const rcsc::WorldModel & wm = agent->world();

    const rcsc::PlayerPtrCont & opps = wm.opponentsFromSelf();
    const rcsc::PlayerObject * nearest_opp
        = ( opps.empty()
            ? static_cast< rcsc::PlayerObject * >( 0 )
            : opps.front() );
    const double nearest_opp_dist = ( nearest_opp
                                      ? nearest_opp->distFromSelf()
                                      : 1000.0 );
    const rcsc::Vector2D nearest_opp_pos = ( nearest_opp
                                             ? nearest_opp->pos()
                                             : rcsc::Vector2D( -1000.0, 0.0 ) );

    rcsc::Vector2D pass_point;

    /* oneman kick strategy */
    double target_dist = 0.0;
    rcsc::AngleDeg target_angle_relative(0.0);
    bool stop = false;

    // test
    // target_dist = 1.0;
    // target_dist = 2.0;
    target_dist = 1.5;
    target_angle_relative = rcsc::AngleDeg( 0.0 );

    // first step : radius-angle coordination system definition
    rcsc::Vector2D myPos = wm.self().pos();
    rcsc::Vector2D goalPos(35.0, 0.0);

    rcsc::Vector2D DeltaGoalBasedMyPos = goalPos - myPos;

    rcsc::AngleDeg myBodyDir = wm.self().body();

    rcsc::AngleDeg myNeckDir = wm.self().neck();

    rcsc::AngleDeg myPosDir = myPos.dir();

    rcsc::AngleDeg goal_angle_relative( 10.0 );
    
    goal_angle_relative = DeltaGoalBasedMyPos.dir() - myBodyDir;


    // second step : basic vector definition


    // --basic parameter--
    double ao = 10.0;
    double ad = -1.0 * 2.0;
    double search_radius = 15.0;
    // --basic parameter-- end

	int n=0;

    rcsc::Vector2D strategyVector = rcsc::Vector2D(0.0, 0.0);

    rcsc::Vector2D offensiveVector = rcsc::Vector2D(0.0, 0.0);
    double offensiveDist = ao;
    rcsc::AngleDeg offensiveDir = goal_angle_relative;

    rcsc::Vector2D defensiveVector = rcsc::Vector2D(0.0, 0.0);
    double defensiveDist = 0.0;
    rcsc::AngleDeg defensiveDir = rcsc::AngleDeg(0.0);


    // third step :
    std::cout << "opps : " << opps.size() << std::endl;
    // std::cout << "front_opp dist : " << opps.front()->distFromSelf() << "	"
    //		 << "front_opp pos : " << opps.front()->pos() << std::endl;
    // std::cout << "nearest_opp dist : " << nearest_opp_dist << "	"
    //		 << "nearest_opp pos : " << nearest_opp_pos << std::endl;
    for( int i = 0; i < opps.size(); i++ ) {
	//std::cout << "dist" << i << " : " << opps[i]->distFromSelf() << std::endl;
	if( opps[i]->distFromSelf() > search_radius ) {
		// defensiveDir -= rcsc::AngleDeg( 90.0 );
    		// defensiveDist = 1.0;
		break;
	}

	n++;

	// std::cout << "dist" << i << " : " << opps[i]->distFromSelf() 
	//	  << "pos" << i << " : " << opps[i]->pos() 
	//	  << std::endl;

	rcsc::Vector2D opp_pos_relative = opps[i]->pos() - myPos;

	rcsc::AngleDeg opp_angle_relative = opp_pos_relative.dir() - goal_angle_relative;
	double opp_dist = opps[i]->distFromSelf();

	std::cout << "dist" << i << " : " << opp_dist 
		  << "ang " << i << " : " << opp_angle_relative
		  << std::endl;
	
	rcsc::AngleDeg opp_angle_relative_ = opp_angle_relative;
/*
	if( opp_angle_relative_.abs() < 30.0 ) {
		 opp_angle_relative_ = ( opp_angle_relative_.degree() > 0.0 ? rcsc::AngleDeg(30.0) : rcsc::AngleDeg(-30.0) );
	}
*/
/*
	if( ( opp_angle_relative_ - goal_angle_relative ).abs() < 30.0 ) {
		
		 opp_angle_relative_ += ( opp_angle_relative_.degree() > 0.0 ? rcsc::AngleDeg(30.0) : rcsc::AngleDeg(-30.0) );
	}
*/

	 defensiveDir += opp_angle_relative_;
	// defensiveDir -= opp_angle_relative_;

	// defensiveDist = 1.0 * (opp_dist/5.0);
	defensiveDist += ad * (search_radius / opp_dist);
    }

    // if( opps.size() != 0 ) {
    //	defensiveDir -= rcsc::AngleDeg( 180.0 );
    //	defensiveDist = 1.0;
    // }

	if( ( defensiveDir - offensiveDir ).abs() < 20.0 ) {
		
		 defensiveDir += ( 1.0  > 0.0 ?
						 rcsc::AngleDeg(90.0) : 
						rcsc::AngleDeg(-90.0) );
	}

	if( n != 0 ) defensiveDist /= n;
	
    //  step : target setting
    offensiveVector.setPolar( offensiveDist, offensiveDir );
    defensiveVector.setPolar( defensiveDist, defensiveDir );

    std::cout << "Vo : ( " << offensiveDist << ", " << offensiveDir << " )	" 
		<< "Vd : ( " << defensiveDist << ", " << defensiveDir << " ) " << std::endl;

    strategyVector = offensiveVector + defensiveVector;

    std::cout << "V : ( " << strategyVector.r() << ", " << strategyVector.dir() << " ) " << std::endl;
 
    target_dist = strategyVector.r();// + 2.0;
    // target_angle_relative = goal_angle_relative;
    target_angle_relative = strategyVector.dir();
    //std::cout 	<< "my position(x,y) : " 
    // 		<< "(" << myPos.x << "," 
    //		<< myPos.y << ")" 
    //		<< std::endl;
    //std::cout << "my body dir : " << myBodyDir << std::endl;

    // final step : do oneman kick
    // rcsc::Body_KickToRelative(  target_dist, 
    //				target_angle_relative, 
    //				stop
    //	 			).execute( agent );
    //rcsc::Body_KickToRelative().execute( agent );

    // target_dist = 1.0;

    //agent->doKick( target_dist, target_angle_relative );

	//rcsc::AngleDeg body_dir_drib_target = target_angle_relative;
	rcsc::Vector2D body_dir_drib_target;// = rcsc::Vector2D( target_dist, target_angle_relative );
	body_dir_drib_target.setPolar( target_dist, target_angle_relative );

	
//	rcsc::Body_Dribble2008( body_dir_drib_target,
 //                                       1.0,
  //                                      rcsc::ServerParam::i().maxDashPower(),
   //                                     2
    //                                    ).execute( agent );

	rcsc::Body_Dribble2008( body_dir_drib_target,
                                        1.0,
                                        rcsc::ServerParam::i().maxDashPower(),
                                        2
                                        ).execute( agent );


    agent->setNeckAction( new rcsc::Neck_TurnToLowConfTeammate() );

    return true;
}
