// -*-c++-*-

/*
 *Copyright:

 Copyright (C) Hidehisa AKIYAMA

 This code is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.

 This code is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this code; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

 *EndCopyright:
 */

/////////////////////////////////////////////////////////////////////

#ifdef HAVE_CONFIG_H
#include <config.h>
#include <iostream>
#endif

#include "role_oneman.h"

#include "strategy.h"

#include "bhv_basic_oneman_kick.h"
#include "bhv_basic_move.h"

#include <rcsc/formation/formation.h>

#include <rcsc/player/player_agent.h>
#include <rcsc/player/intercept_table.h>
#include <rcsc/player/debug_client.h>

#include <rcsc/common/logger.h>
#include <rcsc/common/server_param.h>

#include <rcsc/geom/sector_2d.h>

/*-------------------------------------------------------------------*/
/*!

*/
void
RoleOneman::execute( rcsc::PlayerAgent * agent )
{

	//printf( "oneman execute\n" );
	
	/*
	 * Get my condition
	 */
	double myStamina = agent->world().self().stamina();
	double myEffort = agent->world().self().effort();
	double myRecovery = agent->world().self().recovery();

	rcsc::AngleDeg myNeckAngle = agent->world().self().neck();
	rcsc::Vector2D myPreviousPosition = agent->world().self().posPrev();
	rcsc::Vector2D myLastMove = agent->world().self().lastMove();
	double mySpeed = agent->world().self().speed();
	rcsc::GameTime gametime = agent->world().self().catchTime();
	

	/*
	 * Get current state ( relative )
	 */
	double distToBallR = agent->world().ball().distFromSelf();
	rcsc::AngleDeg angToBallR = agent->world().ball().angleFromSelf();
	rcsc::Vector2D ballPositionR = agent->world().ball().rpos();
	rcsc::Vector2D ballPreviousPositionR = agent->world().ball().rposPrev();
	rcsc::Vector2D ballLastSeenPositionR = agent->world().ball().seenRPos();

	/*
	 * Get Accuracy Indication
	 */
	 int lastSeenCount_Ball = agent->world().ball().seenPosCount();
	 int lostCount_Ball = agent->world().ball().lostCount();

    bool kickable = agent->world().self().isKickable();
    if ( agent->world().existKickableTeammate()
         && agent->world().teammatesFromBall().front()->distFromBall()
         < agent->world().ball().distFromSelf() )
    {
        kickable = false;
    }

    if ( kickable )
    {
        doKick( agent );
    }
    else
    {
        doMove( agent );
    }

}

/*-------------------------------------------------------------------*/
/*!

*/
void
RoleOneman::doKick( rcsc::PlayerAgent * agent )
{
    switch ( Strategy::get_ball_area( agent->world().ball().pos() ) ) {
    case Strategy::BA_CrossBlock:
    case Strategy::BA_Stopper:
    case Strategy::BA_Danger:
    case Strategy::BA_DribbleBlock:
    case Strategy::BA_DefMidField:
    case Strategy::BA_DribbleAttack:
    case Strategy::BA_OffMidField:
    case Strategy::BA_Cross:
    case Strategy::BA_ShootChance:
    default:
        //Bhv_BasicOffensiveKick().execute( agent );
        Bhv_BasicOnemanKick().execute( agent );
	//std::cout << "Strategy " << Strategy::get_ball_area( agent->world().ball().pos() ) << std::endl;
        break;
    }
}

/*-------------------------------------------------------------------*/
/*!

*/
void
RoleOneman::doMove( rcsc::PlayerAgent * agent )
{
    const rcsc::WorldModel & wm = agent->world();

    int ball_reach_step = 0;
    if ( ! wm.existKickableTeammate()
         && ! wm.existKickableOpponent() )
    {
        ball_reach_step
            = std::min( wm.interceptTable()->teammateReachCycle(),
                        wm.interceptTable()->opponentReachCycle() );
    }
    const rcsc::Vector2D base_pos
        = wm.ball().inertiaPoint( ball_reach_step );

    rcsc::Vector2D home_pos
        = formation().getPosition( agent->config().playerNumber(),
                                   base_pos );
    if ( rcsc::ServerParam::i().useOffside() )
    {
        home_pos.x = std::min( home_pos.x, wm.offsideLineX() - 1.0 );
    }

    rcsc::dlog.addText( rcsc::Logger::ROLE,
                        "%s: HOME POSITION=(%.2f, %.2f) base_point(%.1f %.1f)"
                        ,__FILE__,
                        home_pos.x, home_pos.y,
                        base_pos.x, base_pos.y );

    switch ( Strategy::get_ball_area( base_pos ) ) {
    case Strategy::BA_CrossBlock:
    case Strategy::BA_Stopper:
    case Strategy::BA_Danger:
    case Strategy::BA_DribbleBlock:
    case Strategy::BA_DefMidField:
    case Strategy::BA_DribbleAttack:
    case Strategy::BA_OffMidField:
    case Strategy::BA_Cross:
    case Strategy::BA_ShootChance:
    default:
        Bhv_BasicMove( home_pos ).execute( agent );
        break;
    }
}
